@@ -1211,6 +1211,9 @@ void trackspline(xcWidget w, caddr_t clientdata, caddr_t calldata)
    origpt = newspline->ctrl[newspline->cycle];
    newspline->ctrl[newspline->cycle] = newpos;
 
+   /* In case a constrained (parameterized) point has been changed, re-substitute */
+   exprsub(eventmode == EPATH_MODE ? TOGENERIC(EDITPART) : (genericptr)newspline);
+
    calcspline(newspline);
    UDrawEditSpline(newspline);
 
@@ -1664,6 +1667,9 @@ void trackpoly(xcWidget w, caddr_t clientdata, caddr_t calldata)
    origpt = *curpt;
    *curpt = newpos;
 
+   /* In case a constrained (parameterized) point has been changed, re-substitute */
+   exprsub(eventmode == EPATH_MODE ? TOGENERIC(EDITPART) : (genericptr)newpoly);
+
    UDrawPolygon(newpoly);
    printpos(newpos.x, newpos.y);
    areawin->save.x = newpos.x;
