@@ -1315,10 +1315,15 @@ void catmove(int x, int y)
    for (k = 0; k < areawin->selects; k++) {
 
       /* Find number of lobj (may have changed) */
-      for (s2 = 0, spec = xobjs.userlibs[i].instlist; spec != NULL;
+
+      if (lobj == NULL)
+	 s2 = -1;
+      else {
+         for (s2 = 0, spec = xobjs.userlibs[i].instlist; spec != NULL;
 		spec = spec->next, s2++)
-	 if (spec->thisinst == lobj)
-	    break;
+	    if (spec->thisinst == lobj)
+	       break;
+      }
 
       exchobj = SELTOOBJINST(areawin->selectlist + k);
       for (s1 = 0, spec = xobjs.userlibs[i].instlist; spec != NULL;
