@@ -571,17 +571,6 @@ void makenewfontbutton()
    fontnumbers[nfontnumbers - 1] = fontcount - 1;
 }
 
-/*--------------------------------------------------------------*/
-/* Set the menu checkmarks on the font menu			*/
-/*--------------------------------------------------------------*/
-
-void togglefontmark(int fontval)
-{
-   if (fonts[fontval].family != NULL)
-      Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),fontfamily",
-		fonts[fontval].family, 0);
-}
-
 #endif /* TCL_WRAPPER */
 
 /*---------------------------------------------------------------*/
@@ -636,15 +625,10 @@ void togglegrid(u_short type)
       "decimal inches",
       "fractional inches",
       "centimeters",
-      "internal units", NULL      /* (jdk) */
+      "internal units", NULL
    };
 
-   /* Note that Tcl_SetVar() doesn't require double-quotes around the	*/
-   /* string; doing so prevents the menu from recognizing the variable	*/
-   /* value.								*/
-
-   Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),gridstyle",
-	stylenames[type], 0);
+   XcFakeTagCall(xcinterp, 3, "config", "coordstyle", stylenames[type]);
 }
 
 #endif /* TCL_WRAPPER */
@@ -847,23 +831,20 @@ void setcolormark(int colorval)
 {
    /* Set GUI variables and execute any command tags associated */
    /* with the "color" command */
-
-   Tcl_Obj *objv[3];
+  
+   char cstr[5];
    int i;
 
-   objv[0] = Tcl_NewStringObj("color", 5);
-   objv[1] = Tcl_NewStringObj("set", 3);
-   if (colorval == DEFAULTCOLOR)
-      objv[2] = Tcl_NewStringObj("inherit", 7);
-   else {
+   if (colorval != DEFAULTCOLOR) {
       for (i = 0; i < number_colors; i++)
-         if (colorlist[i].color.pixel == colorval)
+         if (colorlist[i].color.pixel == colorval) {
+	    sprintf(cstr, "%5d", i);
 	    break;
-      objv[2] = Tcl_NewIntObj((int)i);
+	 }
    }
-   Tcl_SetVar2Ex(xcinterp, "XCWinOps", "$XCOps(focus),colorval", objv[2], 0);
 
-   XcTagCallback(xcinterp, 3, objv);
+   XcFakeTagCall(xcinterp, 3, "color", "set", (colorval == DEFAULTCOLOR) ?
+	"inherit" : cstr);
 }
 
 /*----------------------------------------------------------------*/
@@ -872,40 +853,25 @@ void setcolormark(int colorval)
 
 void setallstylemarks(u_short styleval)
 {
-   /* Set GUI variables and execute any command tags associated	*/
-   /* with the "fill" and "border" commands.			*/
+   /* Execute any command tags associated	*/
+   /* with the "fill" and "border" commands.	*/
 
-   Tcl_Obj *objv[3];
+   char fstr[10];
    int fillfactor;
    const char *bptr;
 
-   const char *borders[] = {"solid", "unbordered", "dashed", "dotted", NULL}; /* (jdk) */
+   const char *borders[] = {"solid", "unbordered", "dashed", "dotted", NULL};
    enum BorderIdx { SolidIdx, UnborderedIdx, DashedIdx, DottedIdx };
 
-   objv[0] = Tcl_NewStringObj("fill", 4);
-
    if (styleval & FILLED) {
       fillfactor = (int)(12.5 * (float)(1 + ((styleval & FILLSOLID) >> 5)));
-      if (fillfactor == 100)
-         objv[1] = Tcl_NewStringObj("solid", 5);
+      if (fillfactor < 100)
+	 sprintf(fstr, "%d", fillfactor);
       else
-         objv[1] = Tcl_NewIntObj(fillfactor);
+	 strcat(fstr, "solid");
    }
    else
-      objv[1] = Tcl_NewStringObj("unfilled", 8);
-
-   if (styleval & OPAQUE)
-      objv[2] = Tcl_NewStringObj("opaque", 6);
-   else
-      objv[2] = Tcl_NewStringObj("transparent", 11);
-
-   Tcl_SetVar2Ex(xcinterp, "XCWinOps", "$XCOps(focus),fillamount", objv[1], 0);
-   Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),opaque", (styleval & OPAQUE) ?
-		"true" : "false", 0);
-   Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),bboxtype", (styleval & BBOX) ?
-		"true" : "false", 0);
-   Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),polyclosed",
-		(styleval & UNCLOSED) ? "false" : "true", 0);
+      strcat(fstr, "unfilled");
 
    switch (styleval & BORDERS) {
       case DASHED:
@@ -921,9 +887,14 @@ void setallstylemarks(u_short styleval)
 	 bptr = borders[SolidIdx];
 	 break;
    }
-   Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),linestyle", (char *)bptr, 0);
 
-   XcTagCallback(xcinterp, 3, objv);
+   XcFakeTagCall(xcinterp, 3, "fill", fstr,
+		(styleval & OPAQUE) ? "opaque" : "transparent");
+   XcFakeTagCall(xcinterp, 3, "border", "bbox", (styleval & BBOX) ? "true" :
+		"false");
+   XcFakeTagCall(xcinterp, 2, "border", (styleval & UNCLOSED) ? "unclosed" :
+		"closed");
+   XcFakeTagCall(xcinterp, 2, "border", bptr);
 }
 
 #endif /* TCL_WRAPPER */
@@ -1115,6 +1086,16 @@ void setcolor(xcWidget w, int cindex)
    }
 }
 
+/*--------------------------------------------------------------*/
+/* Set the menu checkmarks on the font menu			*/
+/*--------------------------------------------------------------*/
+
+void togglefontmark(int fontval)
+{
+   if (fonts[fontval].family != NULL)
+      XcFakeTagCall(xcinterp, 3, "label", "family", fonts[fontval].family);
+}
+
 /*------------------------------------------------------*/
 /* Set checkmarks on label style menu 			*/
 /* fvalue is for font, jvalue is for justification	*/
@@ -1122,106 +1103,51 @@ void setcolor(xcWidget w, int cindex)
 
 void togglestylemark(int styleval)
 {
-   switch(styleval) {
-      case 0: Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),fontstyle",
-		"normal", 0);
-	 break;
-      case 1: Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),fontstyle",
-		"bold", 0);
-	 break;
-      case 2: Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),fontstyle",
-		"italic", 0);
-	 break;
-      case 3: Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),fontstyle",
-		"bolditalic", 0);
-	 break;
-   }
+   char *cstyle = translatestyle(styleval);
+   if (cstyle != NULL)
+      XcFakeTagCall(xcinterp, 3, "label", "style", cstyle);
 }
 
+/*------------------------------------------------------*/
+/* Set checkmarks on label encoding menu		*/
 /*------------------------------------------------------*/
 
 void toggleencodingmark(int encodingval)
 {
-   switch(encodingval) {
-      case 0: Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),fontencoding",
-		"Standard", 0);
-	 break;
-      case 1: Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),fontencoding",
-		"special", 0);
-	 break;
-      case 2: Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),fontencoding",
-		"ISOLatin1", 0);
-	 break;
-      case 3: Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),fontencoding",
-		"ISOLatin2", 0);
-	 break;
-      case 4: Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),fontencoding",
-		"ISOLatin3", 0);
-	 break;
-      case 5: Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),fontencoding",
-		"ISOLatin4", 0);
-	 break;
-      case 6: Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),fontencoding",
-		"ISOLatin5", 0);
-	 break;
-      case 7: Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),fontencoding",
-		"ISOLatin6", 0);
-	 break;
-   }
+   char *cenc = translateencoding(encodingval);
+   if (cenc != NULL)
+      XcFakeTagCall(xcinterp, 3, "label", "encoding", cenc);
 }
 
+/*------------------------------------------------------*/
+/* Set checkmarks on label justification & flags menu	*/
 /*------------------------------------------------------*/
 
 void togglejustmarks(int justvalue)
 {
-   switch(justvalue & RLJUSTFIELD) {
-      case NORMAL: Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),jhoriz",
-		"left", 0);
-	 break;
-      case NOTLEFT: Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),jhoriz",
-		"center", 0);
-	 break;
-      case RIGHT|NOTLEFT: Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),jhoriz",
-		"right", 0);
-	 break;
-   }
-   switch(justvalue & TBJUSTFIELD) {
-      case NORMAL: Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),jvert",
-		"bottom", 0);
-	 break;
-      case NOTBOTTOM: Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),jvert",
-		"middle", 0);
-	 break;
-      case TOP|NOTBOTTOM: Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),jvert",
-		"top", 0);
-	 break;
-   }
-
-   /* Flip Invariance property */
-   Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),flipinvariant",
-		(justvalue & FLIPINV) ?  "true" : "false", 0);
-
-   /* LaTeX mode property */
-   Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),latexmode",
-		(justvalue & LATEXLABEL) ?  "true" : "false", 0);
-     
-   /* Pin visibility property */
-   Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),pinvisible",
-		(justvalue & PINVISIBLE) ?  "true" : "false", 0);
+   XcFakeTagCall(xcinterp, 4, "label", "justify", (justvalue & RIGHT) ? "right" :
+	(justvalue & NOTLEFT) ? "center" : "left",
+	(justvalue & TOP) ? "top" : (justvalue & NOTBOTTOM) ? "middle" : "bottom");
+   XcFakeTagCall(xcinterp, 3, "label", "flipinvariant", (justvalue & FLIPINV) ?
+	"true" : "false");
+   XcFakeTagCall(xcinterp, 3, "label", "latex", (justvalue & LATEXLABEL) ?
+	"true" : "false");
+   XcFakeTagCall(xcinterp, 3, "label", "visible", (justvalue & PINVISIBLE) ?
+	"true" : "false");
 }
 
-/*------------------------------------------------------*/
+/*-------------------------------------------------------------*/
+/* Simultaneously set all relevant checkmarks for a text label */
+/*-------------------------------------------------------------*/
 
 void setfontmarks(short fvalue, short jvalue)
 {
-   if ((fvalue >= 0) && (fontcount > 0)) {
-      togglestylemark(fonts[fvalue].flags & 0x03);
-      toggleencodingmark((fonts[fvalue].flags & 0xf80) >> 7);
+   if ((fvalue >= 0) && (fvalue < fontcount)) {
+      toggleencodingmark(fvalue);
+      togglestylemark(fvalue);
       togglefontmark(fvalue);
    }
-   if (jvalue >= 0) {
-      togglejustmarks(jvalue);
-   }
+   togglejustmarks(jvalue);
 }
 
 #endif /* TCL_WRAPPER */
@@ -1280,261 +1206,6 @@ void setdefaultfontmarks()
    setfontmarks(areawin->psfont, areawin->justify);
 }
 
-/*--------------------------------------------------------------*/
-/* Toggle a bit in the justification field of a label		*/
-/*--------------------------------------------------------------*/
-
-void dojustifybit(xcWidget w, labelptr settext, short bitfield)
-{
-   if (settext != NULL) {
-      int oldjust = (int)settext->justify;
-
-      undrawtext(settext);
-      settext->justify ^= bitfield;
-      redrawtext(settext);
-      pwriteback(areawin->topinstance);
-
-      register_for_undo(XCF_Justify, UNDO_MORE, areawin->topinstance,
-		(genericptr)settext, oldjust);
-   }
-   else
-      areawin->justify ^= bitfield;
-
-#ifndef TCL_WRAPPER
-   if (w != NULL) {
-      Boolean boolval;
-      if (settext)
-	 boolval = (settext->justify & bitfield) ? 0 : 1;
-      else
-	 boolval = (areawin->justify & bitfield) ? 0 : 1;
-      toggle(w, (pointertype)(-1), &boolval);
-   }
-#endif
-}
-
-/*--------------------------------------------------------------*/
-/* Toggle a pin-related bit in the justification field of a	*/
-/* label.  This differs from dojustifybit() in that the label	*/
-/* must be a pin, and this function cannot change the default	*/
-/* behavior set by areawin->justify.				*/
-/*--------------------------------------------------------------*/
-
-void dopinjustbit(xcWidget w, labelptr settext, short bitfield)
-{
-   if ((settext != NULL) && settext->pin) {
-      int oldjust = (int)settext->justify;
-
-      undrawtext(settext);
-      settext->justify ^= bitfield;
-      redrawtext(settext);
-      pwriteback(areawin->topinstance);
-
-      register_for_undo(XCF_Justify, UNDO_MORE, areawin->topinstance,
-		(genericptr)settext, oldjust);
-
-#ifndef TCL_WRAPPER
-      if (w != NULL) {
-	 Boolean boolval = (settext->justify & bitfield) ? 0 : 1;
-	 toggle(w, (pointertype)(-1), &boolval);
-      }
-#endif
-   }
-}
-
-/*----------------------------------------------------------------*/
-/* Set the justification for the label passed as 3rd parameter	  */
-/*----------------------------------------------------------------*/
-
-void setjust(xcWidget w, pointertype value, labelptr settext, short mode)
-{
-   short newjust, oldjust;
-
-   if (settext != NULL) {
-
-      if (mode == 1)
-         newjust = (settext->justify & (NONJUSTFIELD | TBJUSTFIELD))
-			| value;
-      else
-         newjust = (settext->justify & (NONJUSTFIELD | RLJUSTFIELD))
-			| value;
-
-      if (settext->justify != newjust) {
-	  oldjust = (int)settext->justify;
-          undrawtext(settext);
-          settext->justify = newjust;
-          redrawtext(settext);
-          pwriteback(areawin->topinstance);
-
-          register_for_undo(XCF_Justify, UNDO_MORE, areawin->topinstance,
-		(genericptr)settext, oldjust);
-      }
-   }
-   else {
-      if (mode == 1)
-         newjust = (areawin->justify & (NONJUSTFIELD | TBJUSTFIELD))
-			| value;
-      else
-         newjust = (areawin->justify & (NONJUSTFIELD | RLJUSTFIELD))
-			| value;
-      areawin->justify = newjust;
-   }
-#ifndef TCL_WRAPPER
-   if (w != NULL) togglejustifs(w);
-#endif
-}
-
-/*----------------------------------------------------------------*/
-/* Set vertical justification (top, middle, bottom) on all	  */
-/* selected labels						  */
-/*----------------------------------------------------------------*/
-
-void setvjust(xcWidget w, pointertype value, caddr_t nulldata)
-{
-   short *fselect;
-   labelptr settext;
-   short labelcount = 0;
-
-   if (eventmode == TEXT_MODE || eventmode == ETEXT_MODE) {
-      settext = *((labelptr *)EDITPART);
-      setjust(w, value, settext, 2);
-   }
-   else {
-      for (fselect = areawin->selectlist; fselect < areawin->selectlist +
-            areawin->selects; fselect++) {
-         if (SELECTTYPE(fselect) == LABEL) {
-            labelcount++;
-            settext = SELTOLABEL(fselect);
-            setjust(NULL, value, settext, 2);
-         }
-      }
-      if (labelcount == 0) setjust(w, value, NULL, 2);
-      else unselect_all();
-   }
-}
-
-/*----------------------------------------------------------------*/
-/* Set horizontal justification (left, center, right) on all	  */
-/* selected labels						  */
-/*----------------------------------------------------------------*/
-
-void sethjust(xcWidget w, pointertype value, caddr_t nulldata)
-{
-   short *fselect;
-   labelptr settext;
-   short labelcount = 0;
-
-   if (eventmode == TEXT_MODE || eventmode == ETEXT_MODE) {
-      settext = *((labelptr *)EDITPART);
-      setjust(w, value, settext, 1);
-   }
-   else {
-      for (fselect = areawin->selectlist; fselect < areawin->selectlist +
-            areawin->selects; fselect++) {
-         if (SELECTTYPE(fselect) == LABEL) {
-            labelcount++;
-            settext = SELTOLABEL(fselect);
-            setjust(NULL, value, settext, 1);
-         }
-      }
-      if (labelcount == 0) setjust(w, value, NULL, 1);
-      else unselect_all();
-   }
-}
-
-/*--------------------------------------------------------------*/
-/* Set a justify field bit on all selected labels		*/ 
-/* (flip invariance, latex mode, etc.)				*/
-/*--------------------------------------------------------------*/
-
-void setjustbit(xcWidget w, pointertype value, caddr_t nulldata)
-{
-   short *fselect;
-   labelptr settext;
-   short labelcount = 0;
-
-   if (eventmode == TEXT_MODE || eventmode == ETEXT_MODE) {
-      settext = *((labelptr *)EDITPART);
-      dojustifybit(w, settext, (short)value);
-   }
-   else {
-      for (fselect = areawin->selectlist; fselect < areawin->selectlist +
-            areawin->selects; fselect++) {
-         if (SELECTTYPE(fselect) == LABEL) {
-            labelcount++;
-            settext = SELTOLABEL(fselect);
-            dojustifybit(NULL, settext, (short)value);
-         }
-      }
-      if (labelcount == 0) dojustifybit(w, NULL, (short)value);
-      else unselect_all();
-   }
-}
-
-/*--------------------------------------------------------------*/
-/* Set pin-related bit field of "justify" on all selected pins	*/ 
-/* (e.g., pin visibility)					*/
-/*--------------------------------------------------------------*/
-
-void setpinjustbit(xcWidget w, pointertype value, caddr_t nulldata)
-{
-   short *fselect;
-   labelptr settext;
-
-   if (eventmode == TEXT_MODE || eventmode == ETEXT_MODE) {
-      settext = *((labelptr *)EDITPART);
-      if (settext->pin)
-	 dopinjustbit(w, settext, (short)value);
-   }
-   else {
-      for (fselect = areawin->selectlist; fselect < areawin->selectlist +
-            areawin->selects; fselect++) {
-         if (SELECTTYPE(fselect) == LABEL) {
-            settext = SELTOLABEL(fselect);
-	    if (settext->pin)
-               dopinjustbit(NULL, settext, (short)value);
-         }
-      }
-      unselect_all();
-   }
-}
-
-/*---------------------------*/
-/* Set polygon editing style */
-/*---------------------------*/
-
-#ifdef TCL_WRAPPER
-
-void boxedit(xcWidget w, pointertype value, caddr_t nulldata)
-{
-   switch (value) {
-      case MANHATTAN:
-	 Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),polyedittype",
-		"manhattan", 0);
-	 break;
-      case RHOMBOIDX:
-	 Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),polyedittype",
-		"rhomboidx", 0);
-	 break;
-      case RHOMBOIDY:
-	 Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),polyedittype",
-		"rhomboidy", 0);
-	 break;
-      case RHOMBOIDA:
-	 Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),polyedittype",
-		"rhomboida", 0);
-	 break;
-      case NORMAL:
-	 Tcl_SetVar2(xcinterp, "XCWinOps", "$XCOps(focus),polyedittype",
-		"normal", 0);
-	 break;
-   }
-
-   if (areawin->boxedit == value) return;
-   areawin->boxedit = value;
-}
-
-#endif /* TCL_WRAPPER */
-
 /*----------------------------------------------------------------*/
 /* Pick up font name from _STR2 and pass it to loadfontfile()	  */
 /*----------------------------------------------------------------*/
@@ -1923,7 +1594,6 @@ void fontencoding(xcWidget w, pointertype value, caddr_t nulldata)
          }
       }
       if (labelcount == 0) setfontencoding(w, value, NULL);
-      else unselect_all();
    }
 }
 
