@@ -389,28 +389,33 @@ void setparammarks(genericptr thiselem)
 {
    /* Set GUI variables associated with the "parameter" menu.	*/
 
-  int i; /* , j, paramno; (jdk) */
+   int i;
    oparamptr ops;
    eparamptr epp;
+   Boolean ptest[NUM_PARAM_TYPES];
 
-   /* These match the order of parameter definitions in xcircuit.h */
-   const char *paramvars[] = {"positionparam", "substringparam", "xparam",
-	"yparam", "styleparam", "justparam", "startparam", "endparam",
-	"radiusparam", "minorparam", "rotationparam", "scaleparam",
-			      "linewidthparam", "colorparam", NULL};   /* (jdk) */
+   for (i = 0; i < NUM_PARAM_TYPES; i++)
+      ptest[i] = FALSE;
 
-   /* Reset all variables to false */
-   for (i = 0; i < 14; i++)
-      Tcl_SetVar2(xcinterp, "XCOps", (char *)paramvars[i], "false", 0);
+   XcFakeTagCall(xcinterp, 2, "parameter", "reset");
 
    /* For each parameter declared, set the corresponding Tcl variable */
    if (thiselem != NULL) {
       for (epp = thiselem->passed; epp != NULL; epp = epp->next) {
 	 ops = match_param(topobject, epp->key);
 	 if (ops == NULL) continue;	/* error condition */
-         Tcl_SetVar2(xcinterp, "XCOps", (char *)paramvars[ops->which], "true", 0);
+	 XcFakeTagCall(xcinterp, 3, "parameter", "make", translateparamtype(ops->which));
+	 ptest[ops->which] = TRUE;
       }
    }
+
+   /* Now reset all of those parameters that were not set above.		*/
+   /* Note that the parameters that we want to mark ignore the following types:	*/
+   /* "numeric", "substring", "expression", and "position".			*/
+
+   for (i = P_POSITION_X; i <= P_COLOR; i++)
+      if (ptest[i] != TRUE)
+	 XcFakeTagCall(xcinterp, 3, "parameter", "replace", translateparamtype(i));
 }
 #else
 
@@ -453,10 +458,367 @@ void setparammarks(genericptr thiselem)
 
 #endif
 
+/*------------------------------------------------------*/
+/* This function is like epsubstitute() below it, but	*/
+/* only substitutes those values that are expression	*/
+/* types.  This allows constraints to be applied when	*/
+/* editing elements.					*/
+/*------------------------------------------------------*/
+
+int exprsub(genericptr thiselem)
+{
+   genericptr *pgen;
+   eparamptr epp;
+   int k, ival;
+   oparamptr dps, ops;
+   float fval;
+   XPoint *setpt;
+   char *promoted;
+
+   for (epp = thiselem->passed; epp != NULL; epp = epp->next) {
+      ops = match_param(topobject, epp->key);
+      dps = find_param(areawin->topinstance, epp->key);
+      if (dps != NULL) {
+	 switch(dps->type) {
+	    case XC_EXPR:
+	       if ((promoted = evaluate_expr(topobject, dps, areawin->topinstance))
+			== NULL) continue;
+	       if (sscanf(promoted, "%g", &fval) == 1)
+		  ival = (int)(fval + 0.5);
+	       free(promoted);
+	       if (ELEMENTTYPE(thiselem) == PATH)
+		  k = epp->pdata.pathpt[1];
+	       else
+		  k = epp->pdata.pointno;
+	       if (ops->which == P_POSITION_X) {
+		  switch(thiselem->type) {
+		     case PATH:
+			pgen = TOPATH(&thiselem)->plist + epp->pdata.pathpt[0];
+			if (ELEMENTTYPE(*pgen) == POLYGON) {
+			   setpt = TOPOLY(pgen)->points + k;
+			   setpt->x = ival;
+			}
+			else { /* spline */
+			   TOSPLINE(pgen)->ctrl[k].x = ival;
+			}
+			break;
+		     case POLYGON:
+			setpt = TOPOLY(&thiselem)->points + k;
+			setpt->x = ival;
+			break;
+		     case SPLINE:
+			TOSPLINE(&thiselem)->ctrl[k].x = ival;
+			break;
+		  }
+	       }
+	       else if (ops->which == P_POSITION_Y) {
+		  switch(thiselem->type) {
+		     case PATH:
+			pgen = TOPATH(&thiselem)->plist + epp->pdata.pathpt[0];
+			if (ELEMENTTYPE(*pgen) == POLYGON) {
+			   setpt = TOPOLY(pgen)->points + k;
+			   setpt->y = ival;
+			}
+			else { /* spline */
+			   TOSPLINE(pgen)->ctrl[k].y = ival;
+			}
+			break;
+		     case POLYGON:
+			setpt = TOPOLY(&thiselem)->points + k;
+			setpt->y = ival;
+			break;
+		     case SPLINE:
+			TOSPLINE(&thiselem)->ctrl[k].y = ival;
+			break;
+		  }
+	       }
+	 }
+      }
+   }
+}
+
 /*------------------------------------------------------*/
 /* Make numerical parameter substitutions into an	*/
-/* element.  "thisinst" may be NULL, in which case all	*/
-/* default values are used in the substitution.		*/ 
+/* element.
+/*------------------------------------------------------*/
+
+int epsubstitute(genericptr thiselem, objectptr thisobj, objinstptr pinst,
+   Boolean *needrecalc)
+{
+   genericptr *pgen;
+   eparamptr epp;
+   oparamptr dps, ops;
+   int retval = -1;
+   int k, ival;
+   char *key;
+   float fval;
+   XPoint *setpt;
+   char *promoted;
+
+   for (epp = thiselem->passed; epp != NULL; epp = epp->next) {
+
+      /* Use the parameter from the instance, if available.	*/
+      /* Otherwise, revert to the type of the object.	*/
+      /* Normally they will be the same.			*/
+
+      ops = match_param(thisobj, epp->key);
+      dps = (pinst != NULL) ?  find_param(pinst, epp->key) : ops;
+
+      if (dps != NULL) {
+
+	 /* Get integer and float values.  Promote types if necessary */
+
+	 switch(dps->type) {
+	    case XC_INT:
+	       ival = dps->parameter.ivalue;
+	       fval = (float)(ival);
+	       break;
+	    case XC_FLOAT:
+	       fval = dps->parameter.fvalue;
+	       ival = (int)(fval + 0.5);
+	       break;
+	    case XC_STRING:
+	       promoted = textprint(dps->parameter.string, pinst);
+	       if (sscanf(promoted, "%g", &fval) == 1)
+		  ival = (int)(fval + 0.5);
+	       else
+		  ival = 0;
+	       free(promoted);
+	       break;
+	    case XC_EXPR:
+	       if ((promoted = evaluate_expr(thisobj, dps, pinst)) == NULL) continue;
+	       if (sscanf(promoted, "%g", &fval) == 1)
+		  ival = (int)(fval + 0.5);
+	       free(promoted);
+	       break;
+	 }
+      }
+      else if (ops == NULL)
+	 continue;
+
+      if (IS_OBJINST(thiselem)) {
+	 key = epp->pdata.refkey;
+	 if (key != NULL) {
+	    objinstptr thisinst;
+	    oparamptr refop, newop;
+
+	    thisinst = (objinstptr)thiselem;
+
+	    /* Sanity check: refkey must exist in object */
+	    refop = match_param(thisinst->thisobject, key);
+	    if (refop == NULL) {
+	       Fprintf(stderr, "Error:  Reference key %s does not"
+			" exist in object %s\n",
+			key, thisinst->thisobject->name);
+	       continue;
+	    }
+
+	    /* If an instance value already exists, remove it */
+	    newop = match_instance_param(thisinst, refop->key);
+	    if (newop != NULL)
+	       free_instance_param(thisinst, newop);
+	       
+	    /* Create a new instance parameter */
+	    newop = copyparameter(dps);
+	    newop->next = thisinst->params;
+	    thisinst->params = newop;
+
+	    /* Change the key from the parent to the child */
+	    if (strcmp(ops->key, refop->key)) {
+	       free(newop->key);
+	       newop->key = strdup(refop->key);
+	    }
+	    continue;
+	 }
+      }
+
+      if (ELEMENTTYPE(thiselem) == PATH)
+	 k = epp->pdata.pathpt[1];
+      else
+	 k = epp->pdata.pointno;
+      switch(ops->which) {
+	 case P_POSITION_X:
+	    retval = max(retval, 1);
+	    switch(thiselem->type) {
+	       case PATH:
+		  pgen = TOPATH(&thiselem)->plist + epp->pdata.pathpt[0];
+		  if (ELEMENTTYPE(*pgen) == POLYGON) {
+		     setpt = TOPOLY(pgen)->points + k;
+		     setpt->x = ival;
+		  }
+		  else { /* spline */
+		     TOSPLINE(pgen)->ctrl[k].x = ival;
+		     if (needrecalc) *needrecalc = True;
+		  }
+		  break;
+	       case POLYGON:
+	          setpt = TOPOLY(&thiselem)->points + k;
+		  setpt->x = ival;
+		  break;
+	       case SPLINE:
+		  TOSPLINE(&thiselem)->ctrl[k].x = ival;
+		  if (needrecalc) *needrecalc = True;
+		  break;
+	       case LABEL:
+		  TOLABEL(&thiselem)->position.x = ival;
+		  break;
+	       case OBJINST:
+		  TOOBJINST(&thiselem)->position.x = ival;
+		  break;
+	       case ARC:
+		  TOARC(&thiselem)->position.x = ival;
+		  break;
+	    }
+	    break;
+	 case P_POSITION_Y:
+	    retval = max(retval, 1);
+	    switch(thiselem->type) {
+	       case PATH:
+		  pgen = TOPATH(&thiselem)->plist + epp->pdata.pathpt[0];
+		  if (ELEMENTTYPE(*pgen) == POLYGON) {
+		     setpt = TOPOLY(pgen)->points + k;
+		     setpt->y = ival;
+		  }
+		  else { /* spline */
+		     TOSPLINE(pgen)->ctrl[k].y = ival;
+		     if (needrecalc) *needrecalc = True;
+		  }
+		  break;
+	       case POLYGON:
+	          setpt = TOPOLY(&thiselem)->points + k;
+		  setpt->y = ival;
+		  break;
+	       case SPLINE:
+		  TOSPLINE(&thiselem)->ctrl[k].y = ival;
+		  if (needrecalc) *needrecalc = True;
+		  break;
+	       case LABEL:
+		  TOLABEL(&thiselem)->position.y = ival;
+		  break;
+	       case OBJINST:
+		  TOOBJINST(&thiselem)->position.y = ival;
+		  break;
+	       case ARC:
+		  TOARC(&thiselem)->position.y = ival;
+		  break;
+	    }
+	    break;
+	 case P_STYLE:
+	    retval = max(retval, 0);
+	    switch(thiselem->type) {
+	       case POLYGON:
+		  TOPOLY(&thiselem)->style = ival;
+		  break;
+	       case SPLINE:
+		  TOSPLINE(&thiselem)->style = ival;
+		  break;
+	       case ARC:
+		  TOARC(&thiselem)->style = ival;
+		  break;
+	       case PATH:
+		  TOPATH(&thiselem)->style = ival;
+		  break;
+	    }
+	    break;
+	 case P_JUSTIFY:
+	    retval = max(retval, 1);
+	    switch(thiselem->type) {
+	       case LABEL:
+		  TOLABEL(&thiselem)->justify = ival;
+		  break;
+	    }
+	    break;
+	 case P_ANGLE1:
+	    retval = max(retval, 1);
+	    switch(thiselem->type) {
+	       case ARC:
+		  TOARC(&thiselem)->angle1 = fval;
+		  if (needrecalc) *needrecalc = True;
+		  break;
+	    }
+	    break;
+	 case P_ANGLE2:
+	    retval = max(retval, 1);
+	    switch(thiselem->type) {
+	       case ARC:
+		  TOARC(&thiselem)->angle1 = fval;
+		  if (needrecalc) *needrecalc = True;
+		  break;
+	    }
+	    break;
+	 case P_RADIUS:
+	    retval = max(retval, 1);
+	    switch(thiselem->type) {
+	       case ARC:
+		  TOARC(&thiselem)->radius = ival;
+		  TOARC(&thiselem)->yaxis = ival;
+		  if (needrecalc) *needrecalc = True;
+		  break;
+	    }
+	    break;
+	 case P_MINOR_AXIS:
+	    retval = max(retval, 1);
+	    switch(thiselem->type) {
+	       case ARC:
+		  TOARC(&thiselem)->yaxis = ival;
+		  if (needrecalc) *needrecalc = True;
+		  break;
+	    }
+	    break;
+	 case P_ROTATION:
+	    retval = max(retval, 1);
+	    switch(thiselem->type) {
+	       case LABEL:
+		  TOLABEL(&thiselem)->rotation = ival;
+		  break;
+	       case OBJINST:
+		  TOOBJINST(&thiselem)->rotation = ival;
+		  break;
+	    }
+	    break;
+	 case P_SCALE:
+	    retval = max(retval, 1);
+	    switch(thiselem->type) {
+	       case LABEL:
+		  TOLABEL(&thiselem)->scale = fval;
+		  break;
+	       case OBJINST:
+		  TOOBJINST(&thiselem)->scale = fval;
+		  break;
+	    }
+	    break;
+	 case P_LINEWIDTH:
+	    retval = max(retval, 0);
+	    switch(thiselem->type) {
+	       case POLYGON:
+		  TOPOLY(&thiselem)->width = fval;
+		  break;
+	       case SPLINE:
+		  TOSPLINE(&thiselem)->width = fval;
+		  break;
+	       case ARC:
+		  TOARC(&thiselem)->width = fval;
+		  break;
+	       case PATH:
+		  TOPATH(&thiselem)->width = fval;
+		  break;
+	    }
+	    break;
+	 case P_COLOR:
+	    retval = max(retval, 0);
+	    thiselem->color = ival;
+	    break;
+      }
+   }
+
+   return retval;
+}
+
+/*------------------------------------------------------*/
+/* Make numerical parameter substitutions into all	*/
+/* elements of an object.  "thisinst" may be NULL, in	*/
+/* which case all default values are used in the	*/
+/* substitution.					*/ 
 /*							*/
 /* Return values:					*/
 /*   -1 if the instance declares no parameters		*/
@@ -468,277 +830,16 @@ void setparammarks(genericptr thiselem)
 int opsubstitute(objectptr thisobj, objinstptr pinst)
 {
    genericptr *eptr, *pgen, thiselem;
-   int k, ival; /* i, j, (jdk) */
-   char *key;
-   float fval;
-   oparamptr dps, ops; /* , cps; (jdk) */
-   eparamptr epp;
-   XPoint *setpt;
-   Boolean needrecalc;	/* for arcs and splines */
    int retval = -1;
-   char *promoted;
+   Boolean needrecalc;	/* for arcs and splines */
 
    for (eptr = thisobj->plist; eptr < thisobj->plist + thisobj->parts; eptr++) {
 
       needrecalc = False;
       thiselem = *eptr;
       if (thiselem->passed == NULL) continue;	/* Nothing to substitute */
+      retval = epsubstitute(thiselem, thisobj, pinst, &needrecalc);
 
-      for (epp = thiselem->passed; epp != NULL; epp = epp->next) {
-
-	 /* Use the parameter from the instance, if available.	*/
-	 /* Otherwise, revert to the type of the object.	*/
-	 /* Normally they will be the same.			*/
-
-         ops = match_param(thisobj, epp->key);
-	 dps = (pinst != NULL) ?  find_param(pinst, epp->key) : ops;
-
-	 if (dps != NULL) {
-
-	    /* Get integer and float values.  Promote types if necessary */
-
-	    switch(dps->type) {
-	       case XC_INT:
-		  ival = dps->parameter.ivalue;
-		  fval = (float)(ival);
-		  break;
-	       case XC_FLOAT:
-		  fval = dps->parameter.fvalue;
-		  ival = (int)(fval + 0.5);
-		  break;
-	       case XC_STRING:
-		  promoted = textprint(dps->parameter.string, pinst);
-		  if (sscanf(promoted, "%g", &fval) == 1)
-		     ival = (int)(fval + 0.5);
-		  else
-		     ival = 0;
-		  free(promoted);
-		  break;
-	       case XC_EXPR:
-		  if ((promoted = evaluate_expr(thisobj, dps, pinst)) == NULL) continue;
-		  if (sscanf(promoted, "%g", &fval) == 1)
-		     ival = (int)(fval + 0.5);
-		  free(promoted);
-		  break;
-	    }
-	 }
-	 else if (ops == NULL)
-	    continue;
-
-	 if (IS_OBJINST(thiselem)) {
-	    key = epp->pdata.refkey;
-	    if (key != NULL) {
-	       objinstptr thisinst;
-	       oparamptr refop, newop;
-
-	       thisinst = TOOBJINST(eptr);
-
-	       /* Sanity check: refkey must exist in object */
-	       refop = match_param(thisinst->thisobject, key);
-	       if (refop == NULL) {
-		  Fprintf(stderr, "Error:  Reference key %s does not"
-				" exist in object %s\n",
-				key, thisinst->thisobject->name);
-		  continue;
-	       }
-
-	       /* If an instance value already exists, remove it */
-	       newop = match_instance_param(thisinst, refop->key);
-	       if (newop != NULL)
-		  free_instance_param(thisinst, newop);
-	       
-	       /* Create a new instance parameter */
-	       newop = copyparameter(dps);
-	       newop->next = thisinst->params;
-	       thisinst->params = newop;
-
-	       /* Change the key from the parent to the child */
-	       if (strcmp(ops->key, refop->key)) {
-		  free(newop->key);
-		  newop->key = strdup(refop->key);
-	       }
-	       continue;
-	    }
-	 }
-
-	 if (ELEMENTTYPE(thiselem) == PATH)
-	    k = epp->pdata.pathpt[1];
-	 else
-	    k = epp->pdata.pointno;
-         switch(ops->which) {
-	    case P_POSITION_X:
-	       retval = max(retval, 1);
-	       switch(thiselem->type) {
-		  case PATH:
-		     pgen = TOPATH(&thiselem)->plist + epp->pdata.pathpt[0];
-		     if (ELEMENTTYPE(*pgen) == POLYGON) {
-			setpt = TOPOLY(pgen)->points + k;
-			setpt->x = ival;
-		     }
-		     else { /* spline */
-			TOSPLINE(pgen)->ctrl[k].x = ival;
-			needrecalc = True;
-		     }
-		     break;
-	          case POLYGON:
-	             setpt = TOPOLY(&thiselem)->points + k;
-		     setpt->x = ival;
-		     break;
-	          case SPLINE:
-		     TOSPLINE(&thiselem)->ctrl[k].x = ival;
-		     needrecalc = True;
-		     break;
-	          case LABEL:
-		     TOLABEL(&thiselem)->position.x = ival;
-		     break;
-	          case OBJINST:
-		     TOOBJINST(&thiselem)->position.x = ival;
-		     break;
-	          case ARC:
-		     TOARC(&thiselem)->position.x = ival;
-		     break;
-	       }
-	       break;
-	    case P_POSITION_Y:
-	       retval = max(retval, 1);
-	       switch(thiselem->type) {
-		  case PATH:
-		     pgen = TOPATH(&thiselem)->plist + epp->pdata.pathpt[0];
-		     if (ELEMENTTYPE(*pgen) == POLYGON) {
-			setpt = TOPOLY(pgen)->points + k;
-			setpt->y = ival;
-		     }
-		     else { /* spline */
-			TOSPLINE(pgen)->ctrl[k].y = ival;
-			needrecalc = True;
-		     }
-		     break;
-	          case POLYGON:
-	             setpt = TOPOLY(&thiselem)->points + k;
-		     setpt->y = ival;
-		     break;
-	          case SPLINE:
-		     TOSPLINE(&thiselem)->ctrl[k].y = ival;
-		     needrecalc = True;
-		     break;
-	          case LABEL:
-		     TOLABEL(&thiselem)->position.y = ival;
-		     break;
-	          case OBJINST:
-		     TOOBJINST(&thiselem)->position.y = ival;
-		     break;
-	          case ARC:
-		     TOARC(&thiselem)->position.y = ival;
-		     break;
-	       }
-	       break;
-	    case P_STYLE:
-	       retval = max(retval, 0);
-	       switch(thiselem->type) {
-	          case POLYGON:
-		     TOPOLY(&thiselem)->style = ival;
-		     break;
-	          case SPLINE:
-		     TOSPLINE(&thiselem)->style = ival;
-		     break;
-	          case ARC:
-		     TOARC(&thiselem)->style = ival;
-		     break;
-	          case PATH:
-		     TOPATH(&thiselem)->style = ival;
-		     break;
-	       }
-	       break;
-	    case P_JUSTIFY:
-	       retval = max(retval, 1);
-	       switch(thiselem->type) {
-	          case LABEL:
-		     TOLABEL(&thiselem)->justify = ival;
-		     break;
-	       }
-	       break;
-	    case P_ANGLE1:
-	       retval = max(retval, 1);
-	       switch(thiselem->type) {
-	          case ARC:
-		     TOARC(&thiselem)->angle1 = fval;
-		     needrecalc = True;
-		     break;
-	       }
-	       break;
-	    case P_ANGLE2:
-	       retval = max(retval, 1);
-	       switch(thiselem->type) {
-	          case ARC:
-		     TOARC(&thiselem)->angle1 = fval;
-		     needrecalc = True;
-		     break;
-	       }
-	       break;
-	    case P_RADIUS:
-	       retval = max(retval, 1);
-	       switch(thiselem->type) {
-	          case ARC:
-		     TOARC(&thiselem)->radius = ival;
-		     TOARC(&thiselem)->yaxis = ival;
-		     needrecalc = True;
-		     break;
-	       }
-	       break;
-	    case P_MINOR_AXIS:
-	       retval = max(retval, 1);
-	       switch(thiselem->type) {
-	          case ARC:
-		     TOARC(&thiselem)->yaxis = ival;
-		     needrecalc = True;
-		     break;
-	       }
-	       break;
-	    case P_ROTATION:
-	       retval = max(retval, 1);
-	       switch(thiselem->type) {
-	          case LABEL:
-		     TOLABEL(&thiselem)->rotation = ival;
-		     break;
-	          case OBJINST:
-		     TOOBJINST(&thiselem)->rotation = ival;
-		     break;
-	       }
-	       break;
-	    case P_SCALE:
-	       retval = max(retval, 1);
-	       switch(thiselem->type) {
-	          case LABEL:
-		     TOLABEL(&thiselem)->scale = fval;
-		     break;
-	          case OBJINST:
-		     TOOBJINST(&thiselem)->scale = fval;
-		     break;
-	       }
-	       break;
-	    case P_LINEWIDTH:
-	       retval = max(retval, 0);
-	       switch(thiselem->type) {
-	          case POLYGON:
-		     TOPOLY(&thiselem)->width = fval;
-		     break;
-	          case SPLINE:
-		     TOSPLINE(&thiselem)->width = fval;
-		     break;
-	          case ARC:
-		     TOARC(&thiselem)->width = fval;
-		     break;
-	          case PATH:
-		     TOPATH(&thiselem)->width = fval;
-		     break;
-	       }
-	       break;
-	    case P_COLOR:
-	       retval = max(retval, 0);
-	       thiselem->color = ival;
-	       break;
-         }
-      }
       /* substitutions into arcs and splines require that the	*/
       /* line segments be recalculated.				*/
 
@@ -1831,6 +1932,14 @@ void makeparam(labelptr thislabel, char *key)
    stringpart *begpart, *endpart;
    char *newkey;
 
+   /* Ensure that no two parameters have the same name! */
+
+   if (check_param(topobject, key)) {
+      Wprintf("There is already a parameter named %s!", key);
+      areawin->textend = 0;
+      return;
+   }
+
    /* make sure this does not overlap another parameter */
 
    if (paramcross(topobject, thislabel)) {
@@ -1866,8 +1975,17 @@ void makeparam(labelptr thislabel, char *key)
       endpart = makesegment(&thislabel->string, endpart);
    }
    else {				/* full string */
-      makesegment(&thislabel->string, thislabel->string);
-      begpart = thislabel->string;
+      /* Don't include the first font designator as part of the		*/
+      /* parameter or else havoc ensues.				*/
+      if (thislabel->string->type == FONT_NAME && thislabel->string->nextpart
+		!= NULL) {
+         makesegment(&thislabel->string, thislabel->string->nextpart);
+         begpart = thislabel->string->nextpart;
+      }
+      else {
+         makesegment(&thislabel->string, thislabel->string);
+         begpart = thislabel->string;
+      }
       endpart = makesegment(&thislabel->string, NULL);
    }
    begpart->type = PARAM_START;
