@@ -38,6 +38,11 @@ extern Tcl_Obj *evaluate_raw(objectptr, oparamptr, objinstptr, int *);
 extern char *TCL_to_PS(char *);
 extern XCWindowData *GUI_init(int, Tcl_Obj *CONST objv[]);
 extern void build_app_database(Tk_Window);
+extern int XcFakeTagCall(Tcl_Interp *, int, ...);
+extern char *translateparamtype(int);
+extern char *translatestyle(int);
+extern char *translateencoding(int);
+
 #endif
 
 extern char *evaluate_expr(objectptr, oparamptr, objinstptr);
@@ -268,6 +273,7 @@ extern void varfcheck(FILE *, float, objectptr, short *, genericptr, u_char);
 extern void varpcheck(FILE *, short, objectptr, int, short *, genericptr, u_char);
 extern void getfile(xcWidget, pointertype, caddr_t);
 extern Boolean nextfilename(void);
+extern FILE *libopen(char *, short, char *, int);
 extern void loadfontlib(void);
 extern void loadglib(Boolean, short, short);
 extern void loadulib(void);
@@ -753,9 +759,13 @@ extern genericptr findparam(objectptr, void *, u_char);
 extern Boolean paramcross(objectptr, labelptr);
 extern oparamptr parampos(objectptr, labelptr, char *, short *, short *);
 extern int opsubstitute(objectptr, objinstptr);
+extern int exprsub(genericptr);
+extern int epsubstitute(genericptr, objectptr, objinstptr, Boolean *);
 extern int psubstitute(objinstptr);
 extern Boolean has_param(genericptr);
 extern oparamptr copyparameter(oparamptr);
+extern eparamptr copyeparam(eparamptr, genericptr);
+
 extern void pwriteback(objinstptr);
 extern short paramlen(u_char *);
 extern int natstrlen(u_char *);
