@@ -205,13 +205,15 @@ void quit(xcWidget w, caddr_t nulldata)
    Matrixptr curmatrix, dmatrix;
 
    /* free up CTM Matrix Stack */
-   curmatrix = areawin->MatStack;
-   while (curmatrix != NULL) {
-      dmatrix = curmatrix->nextmatrix;
-      free(curmatrix);
-      curmatrix = dmatrix;
+   if (areawin != NULL) {
+      curmatrix = areawin->MatStack;
+      while (curmatrix != NULL) {
+         dmatrix = curmatrix->nextmatrix;
+         free(curmatrix);
+         curmatrix = dmatrix;
+      }
+      areawin->MatStack = NULL;
    }
-   areawin->MatStack = NULL;
 
    /* free the colormap if a new one has been installed */
 
@@ -385,7 +387,7 @@ void quitcheck(xcWidget w, caddr_t clientdata, caddr_t calldata)
    }
    else {
       free(promptstr);
-      quit(areawin->area, NULL);
+      quit(w, NULL);
    }
 }
 
@@ -970,7 +972,7 @@ void delete_window(XCWindowData *window)
    XCWindowData *searchwin, *lastwin = NULL;
 
    if (xobjs.windowlist->next == NULL) {
-      quitcheck(NULL, NULL, NULL);
+      quitcheck((window == NULL) ? NULL : window->area, NULL, NULL);
       return;
    }
 
