@@ -413,6 +413,7 @@ enum loadmodes {IMPORT = 1, PSBKGROUND, SCRIPT, RECOVER,
 /* Library types							*/
 /*----------------------------------------------------------------------*/
 
+#define FONTENCODING   -1  /* Used only by libopen() */
 #define FONTLIB		0
 #define PAGELIB		1
 #define LIBLIB		2
@@ -541,22 +542,25 @@ typedef XfPoint* fpointlist;
 /* Allowed parameterization types					*/
 /*----------------------------------------------------------------------*/
 
-#define P_NUMERIC	0	/* uncommitted numeric parameter */
-#define P_SUBSTRING	1
-#define P_POSITION_X	2
-#define P_POSITION_Y	3
-#define P_STYLE		4
-#define P_JUSTIFY	5
-#define P_ANGLE1	6
-#define P_ANGLE2	7
-#define P_RADIUS	8
-#define P_MINOR_AXIS	9
-#define P_ROTATION	10
-#define P_SCALE		11
-#define P_LINEWIDTH	12
-#define P_COLOR		13
-#define P_EXPRESSION	14
-#define P_POSITION	15	/* mode only, not a real parameter */
+enum paramwhich {
+   P_NUMERIC = 0,	/* uncommitted numeric parameter */
+   P_SUBSTRING,
+   P_POSITION_X,
+   P_POSITION_Y,
+   P_STYLE,
+   P_JUSTIFY,
+   P_ANGLE1,
+   P_ANGLE2,
+   P_RADIUS,
+   P_MINOR_AXIS,
+   P_ROTATION,
+   P_SCALE,
+   P_LINEWIDTH,
+   P_COLOR,
+   P_EXPRESSION,
+   P_POSITION, 		/* mode only, not a real parameter */
+   NUM_PARAM_TYPES
+};
 
 /*----------------------------------------------------------------------*/
 /* Labels are constructed of strings and executables 			*/
